//+------------------------------------------------------------------+
//|                                                                  |
//|  R-FTLM-STLM-Adaptive v.1.0                                      |
//|  Copyright  2009 Riccardo "Rich" Cap                            |
//|  email: richcap.rc@gmail.com                                     |
//|                                                                  |
//|                                                                  |
//|  This indicator draws  FTLM and STLM curves re-calculated over a | 
//|  window of length 'length' every 'filterPersistenceBars' bars    |
//|  Optionally curves can be joined with an interpolation           |
//|  This indicator needs 'R-FATL-SATL-Adaptive' and                 |
//|  'R-MESA_Cutoff_frequencies' indicators as well as               |
//|  R-MESA.ex4 and R-DF.ex4 libraries installed                     |
//|                                                                  |
//|  Permission to use, copy, modify, and distribute this software   |
//|  for any purpose without fee is hereby granted, provided that    |
//|  this entire notice is included in all copies of any software    |
//|  which is or includes a copy or modification of this software    |
//|  and in all copies of the supporting documentation for such      |
//|  software.                                                       |
//|  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS    |
//|  OR IMPLIED WARRANTY.  IN PARTICULAR, NEITHER THE AUTHOR DOES    |
//|  NOT MAKE ANY REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING  |
//|  THE MERCHANTABILITY OF THIS SOFTWARE OR ITS FITNESS FOR ANY     |
//|  PARTICULAR PURPOSE.                                             |
//|                                                                  |
//+------------------------------------------------------------------+


#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Red            // FTLM
#property indicator_color2 Orange         // STLM


//---- buffers
double Buffer1[];
double Buffer2[];

// External parameters 
extern int degree=150;                 // Degree of autocorrelation for MESA (Maximum Entropy Spectrum Analysis)
extern int length=400;                 // Number of previous Bars to perform MESA calculation
extern double max_period=140;          // Upper bound for SATL cutoff period (1/frequency)
extern double min_period=20;           // Lower bound for FATL cutoff period
extern double satl_min_period=60.0;    // Lower bound for SATL (upper bound for FATL is 80% of "*")
extern datetime initial_time=D'1970.01.01 00:00'; // if backwardBars=0, get it from initial_time
extern int filterPersistenceBars=100;   // The SATL and FATL cutoff periods are re-calculated every "*" bars
extern int backwardBars=0;             // The indicator is drawn backard "*" bars       ** Check if == 0 ** (MESA_Cutoff_frequencies problems??

extern bool rectify=false;              // by Default fatl/satle curves are rectified to be continuous between two different filter sets
                                       // by putting to false non-rectified curves are plotted. In this case must be paid attention
                                       // with strategies that look at crossovers

static datetime prevtime=0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   // sets number of initial bars   
   if (backwardBars==0)
   {
      if (initial_time == D'1970.01.01 00:00')
         backwardBars=0;
      else
         backwardBars=iBarShift(NULL,0,initial_time, true)-1;        
   }   
   if (backwardBars==-1)
   {
      Print ("Initial time not valid");
      return (20);            
   }
   else  Print("shift of bar with open time ",TimeToStr(initial_time)," is ",backwardBars);
   
   //---- indicators
   IndicatorDigits(Digits+2);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexDrawBegin(0,0);
   SetIndexDrawBegin(1,0);
   SetIndexBuffer(0,Buffer1);
   SetIndexBuffer(1,Buffer2);
   SetIndexLabel(0,"FTLM");
   SetIndexLabel(1,"STLM");
   
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{ 
   int i, j, counted_bars=IndicatorCounted();

   if(!newPeriodOpening())
   {
      Print("Not the beginning of a new period");
      return(0);  // Not the beginning of a new period
   } 

   // DRAW FTLM and STLM
   
   if(Bars<=backwardBars)
   {
      Print("Too many Bars backwards, please stay under ",Bars," Bars");
      return(0);
   }   
   i=backwardBars;        
   if (counted_bars>Bars-backwardBars-2) 
      i=Bars-counted_bars-2;
      
   while(i>=0)
   {
      double fatl=iCustom(NULL,0,"R-FATL-SATL-Adaptive",degree,length,max_period,min_period,satl_min_period,initial_time,filterPersistenceBars,backwardBars,rectify,0,i+1); // check if shift must be 0 or 1
      double rftl=iCustom(NULL,0,"R-FATL-SATL-Adaptive",degree,length,max_period,min_period,satl_min_period,initial_time,filterPersistenceBars,backwardBars,rectify,1,i+1); // check if shift must be 0 or 1
      double satl=iCustom(NULL,0,"R-FATL-SATL-Adaptive",degree,length,max_period,min_period,satl_min_period,initial_time,filterPersistenceBars,backwardBars,rectify,2,i+1); // check if shift must be 0 or 1
      double rstl=iCustom(NULL,0,"R-FATL-SATL-Adaptive",degree,length,max_period,min_period,satl_min_period,initial_time,filterPersistenceBars,backwardBars,rectify,3,i+1); // check if shift must be 0 or 1
   
      //Print ("fatl: ",fatl," - rftl: ",rftl," - satl: ",satl, " - rstl: ",rstl);
      //Print ("counted bars: ",counted_bars, " - i: ",i);   
      if (rftl < 1000000.0)
         Buffer1[i+1]= fatl - rftl;
      if (rstl < 1000000.0)
         Buffer2[i+1]= satl - rstl;
      i--;
   }
   return(0);
}
  
  

bool newPeriodOpening()
{
   if(prevtime == Time[0]) return(false);
   prevtime = Time[0];
   return(true);
}
  
   
//-------------------------------------------------------------------+